# Juicy

Juicy extends JavaScript's Number, Object, Array and String with some useful methods.

# Installation

*Browsers:*

Just include juicy.min.js:

```html
<srcipt src="juicy.min.js"></script>
```

*Node.js:*

Install with:

```
npm install juicy
```

And,

```coffee-script
require 'juicy'
```

# Usage

### Function

##### include

```coffee-script
class SomeExtension
	class SomeExtension.ClassMethods
		someClassMethod: ->
	
	class SomeExtension.InstanceMethods
		someInstanceMethod: ->

class SomeModel

SomeModel.include SomeExtension

SomeModel.someClassMethod()

model = new SomeModel
model.someInstanceMethod()
```

### Array

##### collect

```coffee-script
items = ['a', 'b'].collect (item) ->
	"#{ item }!"

# items = ['a!', 'b!']
```

##### combination

```coffee-script
['a', 'b', 'c'].combination(1) # [['a'], ['b'], ['c']]

['a', 'b', 'c'].combination(2) # [['a', 'b'], ['c']]
```

##### compact

```coffee-script
[true, false, 'a', null, 'b', undefined].compacted # [true, false, 'a', 'b']
```

##### merge

```coffee-script
['a', 'b'].merge ['c', 'd'] # ['a', 'b', 'c', 'd']
```

##### count

```coffee-script
count = [1, 2, 3, 4].count (item) ->
	item < 3

# count = 2
```

##### remove

```coffee-script
items = ['a', 'b', 'c', 'a']
items.remove 'a'

# items = ['b', 'c']
```

##### removeAt

```coffee-script
items = ['a', 'b', 'c']
items.removeAt 0

# items = ['b', 'c']
```

##### removeIf

```coffee-script
items = ['a', 'b', 'c'].removeIf (item) ->
	item > 'a'

# items = ['a']
```

##### drop

```coffee-script
items = ['a', 'b', 'c']
items.drop 2

# items = ['c']
```

##### dropWhile

```coffee-script
items = ['a', 'b', 'c', 'd'].dropWhile (item) ->
	item < 'd'

# items = ['d']
```

##### eachIndex

```coffee-script
['a', 'b', 'c'].eachIndex (index) ->
	console.log index

# output: 0, 1, 2
```

##### eql

```coffee-script
first = ['a', ['b']]
second = ['a', ['b']]
third = ['a', 'b']

first.eql(second) # true
second.eql(third) # false
```

##### flatten

```coffee-script
['a', ['b']].flattened # ['a', 'b']
```

##### take

```coffee-script
['a', 'b', 'c'].take(2) # ['a', 'b']
```

##### takeWhile

```coffee-script
items = ['a', 'b', 'c'].takeWhile (item) ->
	item < 'c'

# items = ['a', 'b']
```

##### transpose

```coffee-script
[rows, columns] = [['a', 'c'], ['b', 'd']].transposed

# rows[0] = 'a', rows[1] = 'b'
# columns[0] = 'c', columns[1] = 'd'
```

##### first

```coffee-script
items = ['a', 'b']

items.first # 'a'

items.first = 'c'
# items = ['c', 'd']
```

##### last

```coffee-script
items = ['a', 'b']
items.last # 'b'

items.last = 'c'
# items = ['a', 'c']
```

##### shuffle

```coffee-script
items = ['a', 'b']

items.shuffled // or items.shuffle()
```

### Object

##### merge

```coffee-script
first=
	a: 1

second=
	b: 2

Object.merge(first, second) # { a: 1, b: 2 }
```

##### eql

```coffee-script
first=
	a: 1

second=
	a: 1

third=
	b: 2

Object.eql(first, second) # true
Object.eql(second, third) # false
```

### String

##### capitalize

```coffee-script
'hello'.capitalized # 'Hello'
```

##### pluralize

```coffee-script
'item'.pluralized # 'items'
```

##### camelize

```coffee-script
'nice\_product'.camelized 'NiceProduct'
```

##### underscore

```coffee-script
'NiceProduct'.underscored # 'nice\_product'
```

##### trim

```coffee-script
'   hello  '.trimmed # 'hello'
```

##### singularize

```coffee-script
'items'.singularized # 'item'
```

##### humanize

```coffee-script
'nice\_product'.humanized 'Nice product'
```

##### dasherize

```coffee-script
'nice product'.dasherized # 'nice-product'
```

##### titleize

```coffee-script
'nice product'.titleized # 'Nice Product'
```

##### demodulize

```coffee-script
'ActiveController::Metal'.demodulized # 'Metal'
```

##### tableize

```coffee-script
'author id'.tableized # 'author ids'
```

# Tests

You can run tests on Node.js by running ```mocha```, or in browser by opening **test/test.html**.


# License

(The MIT License)

Copyright (c) 2011 Vadim Demedes sbioko@gmail.com

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the 'Software'), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE